import { Component, Vue } from 'vue-property-decorator'
import personalIdGenerator from '@/classes/Service/Generators/PersonalIdGenerator'
import ibanGenerator from '@/classes/Service/Generators/IbanGenerator'
import phoneGenerator from '@/classes/Service/Generators/PhoneGenerator'
import firstNameGenerator from '@/classes/Service/Generators/FirstNameGenerator'
import lastNameGenerator from '@/classes/Service/Generators/LastNameGenerator'
import { Person } from '@/classes/DTO/Person'
import VueClipboard from 'vue-clipboard2'
import { Target } from '@/classes/Enum/Target'
import sender from '@/classes/Service/Browser/Chrome/Sender'
import { Action } from '@/classes/Enum/Action'
import Mocks from '@/components/mocks/main.vue'
import countryResolver from '@/api/CountryResolver'

Vue.use(VueClipboard)

@Component({
  components: {
    Mocks
  }
})
export default class PersonTab extends Vue {
  personalCode = ''
  person = new Person()
  copyText = ''

  async mounted () {
    this.person.country = await countryResolver.resolve()
  }

  get allowedMonths () {
    const result = []
    for (let i = 1; i <= 12; i++) {
      result.push(i)
    }
    return result
  }

  get allowedYears () {
    const result = []
    const currentYear = (new Date()).getFullYear()
    for (let i = 1960; i <= currentYear; i++) {
      result.push(i)
    }
    return result
  }

  copyPersonalCode () {
    const personalCode = personalIdGenerator.generate(this.person)
    this.personalCode = personalCode
    this.copy(personalCode)
  }

  copyIban () {
    const iban = ibanGenerator.generate(this.person.country)
    this.personalCode = ''
    this.copy(iban)
  }

  copyPhone () {
    const phone = phoneGenerator.generate(this.person.country)
    this.personalCode = ''
    this.copy(phone)
  }

  copy (text: string) {
    this.copyText = text
    this.$copyText(text)
  }

  autocomplete () {
    const personalCode = personalIdGenerator.generate(this.person)
    this.personalCode = personalCode
    this.copy(personalCode)
    this.insertValueToNode(Target.PERSONAL_ID, personalCode)
    const iban = ibanGenerator.generate(this.person.country)
    this.insertValueToNode(Target.IBAN, iban)
    const phone = phoneGenerator.generate(this.person.country)
    this.insertValueToNode(Target.PHONE, phone)
    const firstName = firstNameGenerator.generate(this.person.country, this.person.gender)
    this.insertValueToNode(Target.FIRST_NAME, firstName)
    const lastName = lastNameGenerator.generate(this.person.country)
    this.insertValueToNode(Target.LAST_NAME, lastName)
  }

  insertValueToNode (target: Target, value: string): void {
    sender.send({
      action: Action.INSERT,
      target: target,
      value: value
    })
  }
}
